from dnevnikru import settings
from dnevnikru.exceptions import DnevnikError

from bs4 import BeautifulSoup
from typing import Optional, Union
from datetime import date, timedelta, datetime


class Parser:
    @staticmethod
    def last_page(response: str) -> Optional[int]:
        """Функция для получения номера последней страницы (если она есть)"""
        try:
            soup = BeautifulSoup(response, 'lxml')
            all_pages = soup.find('div', {'class': 'pager'})
            pages = all_pages.find_all('li')
            last_page = pages[-1].text
            return last_page
        except AttributeError:
            return None

    @staticmethod
    def save_content(response: str, class2: str) -> tuple:
        """Функция парсинга и сохранения таблиц с сайта"""
        soup = BeautifulSoup(response, 'lxml')
        table = soup.find('table', {'class': class2})
        content = []
        all_rows = table.findAll('tr')
        for row in all_rows:
            content.append([])
            all_cols = row.findAll('td')
            for col in all_cols:
                the_strings = [str(s) for s in col.findAll(text=True)]
                the_text = ''.join(the_strings)
                content[-1].append(the_text)
        content = [a for a in content if a != []]
        return tuple(content)

    @staticmethod
    def get_week_response(session, school: Union[int, str], weeks: int) -> str:
        """Функция для получения html страницы с результатами недели"""
        link = settings.WEEK_LINK
        data_response = session.get(link).text
        day = datetime.strptime(settings.DATEFROM, "%d.%m.%Y") + timedelta(7 * weeks)
        weeks_list = []
        week = date(2021, 7, 19)
        for _ in range(35):
            week = week + timedelta(7)
            weeks_list.append(week.strftime("%d.%m.%Y"))
        for i in weeks_list:
            if day <= datetime.strptime(i, "%d.%m.%Y"):
                week = weeks_list[weeks_list.index(i) - 1]
                break
        soup = BeautifulSoup(data_response, 'lxml')
        user_id = soup.find('option')["value"]
        link = "https://dnevnik.ru/currentprogress/result/{}/{}/{}/{}?UserComeFromSelector=True".format(
            user_id, school, settings.STUDYYEAR, week)
        week_response = session.get(link).text
        return week_response

    @staticmethod
    def get_homework(self, link: str, last_page: Union[str, int], homework_response: str) -> dict:
        """Функция для получения домашних заданий"""
        if last_page is not None:
            subjects = []
            for page in range(1, int(last_page) + 1):
                homework_response = self._main_session.get(link + f"&page={page}", headers={"Referer": link}).text
                for i in Parser.save_content(homework_response, class2='grid gridLines vam hmw'):
                    subject = [i[2], i[0].strip(),
                               " ".join([_.strip() for _ in i[3].split()])]
                    subjects.append(tuple(subject))
            return {"homeworkCount": len(subjects), "homework": tuple(subjects)}
        if last_page is None:
            try:
                subjects = []
                for i in Parser.save_content(homework_response, class2='grid gridLines vam hmw'):
                    subject = [i[2], i[0].strip(),
                               " ".join([_.strip() for _ in i[3].split()])]
                    subjects.append(tuple(subject))
                return {"homeworkCount": len(subjects), "homework": tuple(subjects)}
            except Exception as e:
                raise DnevnikError(e, "DnevnikError")

    @staticmethod
    def get_marks(marks_response: str) -> tuple:
        """Функция для получения оценок"""
        try:
            marks = Parser.save_content(response=marks_response, class2='grid gridLines vam marks')
            for mark in marks:
                mark[2] = mark[2].replace(" ", "")
            return tuple(marks)
        except Exception as e:
            raise DnevnikError(e, "DnevnikError")

    @staticmethod
    def search_people(self, last_page: Union[int, str], link: str, searchpeople_response: str) -> dict:
        """Функция для поиска людей по школе"""
        if last_page is not None:
            members = []
            for page in range(1, int(last_page) + 1):
                members_response = self._main_session.get(link + f"&page={page}").text
                for content in Parser.save_content(members_response, class2='people grid'):
                    member = [content[1].split('\n')[1], content[1].split('\n')[2]]
                    members.append(tuple(member))
            return {"peopleCount": len(members), "people": tuple(members)}
        if last_page is None:
            members = []
            try:
                for content in Parser.save_content(searchpeople_response, class2='people grid'):
                    member = [content[1].split('\n')[1], content[1].split('\n')[2]]
                    members.append(tuple(member))
                return {"peopleCount": len(members), "people": tuple(members)}
            except Exception as e:
                raise DnevnikError(e, "DnevnikError")

    @staticmethod
    def get_birthdays(self, birthdays_response: str, link: str) -> dict:
        """Функция для поиска дней рождений по школе"""
        if "в школе именинников нет." in birthdays_response:
            return {"peopleCount": 0, "people": ()}
        last_page = Parser.last_page(birthdays_response)

        if last_page is not None:
            birthdays = []
            for page in range(1, int(last_page) + 1):
                birthdays_response = self._main_session.get(link + f"&page={page}").text
                for i in Parser.save_content(birthdays_response, class2='people grid'):
                    birthdays.append(i[1].split('\n')[1])
            return {"birthdaysCount": len(birthdays), "birthdays": tuple(birthdays)}
        if last_page is None:
            birthdays = []
            try:
                for i in Parser.save_content(birthdays_response, class2='people grid'):
                    birthdays.append(i[1].split('\n')[1])
                return {"birthdaysCount": len(birthdays), "birthdays": tuple(birthdays)}
            except Exception as e:
                raise DnevnikError(e, "DnevnikError")

    @staticmethod
    def get_week(self, info: str, weeks: int) -> dict:
        """Функция для получения результатов недели"""
        head = "current-progress-{}".format(info)
        item = "current-progress-{}__item"
        item = item.format("list") if info != "schedule" else item.format("schedule")
        week_response = Parser.get_week_response(session=self._main_session,
                                                 school=self._school, weeks=weeks)
        week = {}
        soup = BeautifulSoup(week_response, 'lxml')
        student = soup.findAll("h5", {"class": "h5 h5_bold"})[0].text
        h = soup.find_all("div", {"class": head})[0]
        all_li = h.findAll("li", {"class": item})
        if info == "schedule":
            for li in all_li:
                day = li.find("div").text
                schedule = li.findAll("li")
                schedule = [x.text for x in schedule]
                week.update({day: tuple(schedule)})
            return {"student": student, "schedule": week}
        else:
            week = [i.replace("\n", " ").strip(" ") for i in [i.text for i in all_li]]
            return {"student": student, info: tuple(week)}



from dnevnikru.exceptions import DnevnikError
from dnevnikru.parsers import Parser
from dnevnikru import settings

from datetime import timedelta, datetime
from typing import Union
import requests


class Dnevnik:
    """Базовый класс Дневника"""

    def __init__(self, login: str, password: str) -> None:
        self.__login, self.__password = login, password
        self._main_session = requests.Session()
        self._main_session.headers.update(settings.USER_AGENT)
        self._main_session.post('https://login.dnevnik.ru/login',
                                data={"login": self.__login, "password": self.__password})
        if self._main_session.cookies.get("t0"):
            self._school = self._main_session.cookies.get("t0")
            return
        raise DnevnikError('Authorization error', 'LoginError')

    def homework(self, datefrom=settings.DATEFROM, dateto=settings.DATETO, studyyear=settings.STUDYYEAR,
                 days: int = 10) -> dict:
        """Method for getting homework"""
        # Checking the correctness of arguments
        if datefrom != settings.DATEFROM or days != 10:
            days_count = datetime.strptime(datefrom, '%d.%m.%Y')
            dateto = (days_count + timedelta(days=days)).strftime("%d.%m.%Y")
        if len(datefrom) != 10 or len(dateto) != 10:
            raise DnevnikError("Invalid dateto or datefrom", "Arguments error")
        if str(studyyear) not in datefrom:
            raise DnevnikError("StudyYear must match datefrom", "Arguments error")

        # Get homework
        link = settings.HW_LINK.format(self._school, studyyear, datefrom, dateto)
        homework_response = self._main_session.get(link, headers={"Referer": link}).text
        if "Домашних заданий не найдено." in homework_response:
            return {"homeworkCount": 0, "homework": ()}
        last_page = Parser.last_page(homework_response)
        return Parser.get_homework(self, link=link, last_page=last_page, homework_response=homework_response)

    def marks(self, index: Union[str, int] = "", period: Union[str, int] = "") -> tuple:
        """Method for getting marks"""
        # Get marks
        link = settings.MARKS_LINK.format(self._school, index, str(period))
        marks_response = self._main_session.get(link, headers={"Referer": link}).text
        return Parser.get_marks(marks_response=marks_response)

    def searchpeople(self, group: str = "", name: str = "", grade: Union[str, int] = "") -> dict:
        """Method for getting people from user's school"""
        # Checking the correctness of arguments
        assert group in settings.PEOPLE_GROUPS, "Неверная группа!"

        # Get people
        link = settings.SEARCHPEOPLE_LINK.format(self._school, group, name, str(grade))
        searchpeople_response = self._main_session.get(link).text
        if "Никого не найдено. Измените условия поиска." in searchpeople_response:
            return {"peopleCount": 0, "people": ()}
        last_page = Parser.last_page(searchpeople_response)
        return Parser.search_people(self, last_page=last_page, link=link, searchpeople_response=searchpeople_response)

    def birthdays(self, day: int = settings.DAY, month: int = settings.MONTH, group: str = "") -> dict:
        """Method for getting birthdays"""
        # Checking the correctness of arguments
        assert group in settings.BIRTHDAYS_GROUPS, "Неверная группа!"
        assert day in list(range(1, 32)) or month not in list(range(1, 13)), "Неверный день или месяц!"

        # Get birthdays
        link = settings.BIRTHDAYS_LINK.format(self._school, day, month, group)
        birthdays_response = self._main_session.get(link).text
        return Parser.get_birthdays(self, birthdays_response=birthdays_response, link=link)

    def week(self, info: str = "schedule", weeks: int = 0) -> dict:
        """Method for getting week"""
        # Checking the correctness of arguments
        assert info in settings.WEEK_INFORMATION, 'Invalid info'

        # get week
        return Parser.get_week(self, info=info, weeks=weeks)



from datetime import date, timedelta, datetime
import urllib.parse

DATEFROM = date.today().strftime("%d.%m.%Y")
DATETO = (date.today() + timedelta(days=10)).strftime("%d.%m.%Y")
STUDYYEAR = date.today().strftime("%Y")
DAY = date.today().day
MONTH = date.today().month
CHOOSE = urllib.parse.quote("Показать")
BASE_LINK = "https://schools.dnevnik.ru/"
HW_LINK = "".join(
    (BASE_LINK, "homework.aspx?school={}&tab=&studyYear={}&subject=&datefrom={}&dateto={}&choose=", CHOOSE))
MARKS_LINK = "".join((BASE_LINK, "marks.aspx?school={}&index={}&tab=period&period={}&homebasededucation=False"))
SEARCHPEOPLE_LINK = "".join((BASE_LINK, "school.aspx?school={}&view=members&group={}&filter=&search={}&class={}"))
BIRTHDAYS_LINK = "".join((BASE_LINK, "birthdays.aspx?school={}&view=calendar&action=day&day={}&month={}&group={}"))
WEEK_LINK = "https://dnevnik.ru/currentprogress/choose?userComeFromSelector=True"
USER_AGENT = {"User-Agent": "Mozilla/5.0 (Wayland; Linux x86_64) AppleWebKit/537.36 ("
                            "KHTML, like Gecko) Chrome/94.0.4606.72 Safari/537.36"}
PEOPLE_GROUPS = ['all', 'students', 'staff', 'director', 'management', 'teachers', 'administrators', ""]
BIRTHDAYS_GROUPS = ['all', 'students', 'staff', 'class', '']
WEEK_INFORMATION = ["themes", "attendance", "marks", "schedule", "homeworks"]
